#include <msp430f249.h>

typedef unsigned char BYTE;
typedef unsigned int  WORD;
typedef unsigned long LONG;

//#define USE_XT2_FOR_SMCLK_DEFAULT

LONG  MSP430_GetDCOFreq();

volatile LONG result = 0;

void main(void)
{
  WORD i;
  
  WDTCTL = WDTPW + WDTHOLD; // Stop WDT

  BCSCTL1 = CALBC1_1MHZ;                    // Set DCO
  DCOCTL = CALDCO_1MHZ;

  // turn on XT2 
  BCSCTL1 &= ~XT2OFF;  
   
  // set XT2 frequency range and XCAP bits
  BCSCTL3 |= XT2S_2 + XCAP_3;

  // Wait for xtal to stabilize
  do
  {
    IFG1 &= ~OFIFG;                           // Clear OSCFault flag
    for (i = 0x47FF; i > 0; i--);             // Time for flag to set
  }
  while ((IFG1 & OFIFG));                   // OSCFault flag still set?

#ifdef USE_XT2_FOR_SMCLK_DEFAULT
  // use XT2 as MCLK and SMCLK
  BCSCTL2 |= SELM_2 + SELS;
#else
  // use XT2 as MCLK and DCO as SMCLK
  BCSCTL2 |= SELM_2;
#endif

  // output ACLK, MCLK, SMCLK
  P5SEL |= BIT4 + BIT5 + BIT6;
  P5DIR |= BIT4 + BIT5 + BIT6;

  IE1 |= OFIE;                              // Enable osc fault interrupt

  while(1)
  {
    result = MSP430_GetDCOFreq();
    
    __no_operation();
  }
}

//-----------------------------------------------------------------------------------------------
// GetDCOFreq
// Utilizza il TimerB e ACLK(32KHz) per misurare la frequenza del DCO (torna valore in Hz)
//-----------------------------------------------------------------------------------------------
LONG  MSP430_GetDCOFreq()
{
    WORD  n1,n2;

#ifdef USE_XT2_FOR_SMCLK_DEFAULT
    BYTE  SmclkFromXt2;
    
    // Setta un flag se SMCLK  ricavato da XT2
    if (BCSCTL2 & SELS) SmclkFromXt2=1; else SmclkFromXt2=0;

    // Ricava il clock per SMCLK da DCO
    BCSCTL2 &= ~SELS;
#endif

    // Accende TimerB: conteggio continuo a 16 bit, clock = SMCLK
    TBCTL = TBSSEL1 | MC1 | TBCLR;
    // Capture da CC16B=ACLK, rising edge
    TBCCTL6 = CM1 | CCIS0 | SCS | CAP;
    // Attende il primo evento di capture
    while ( (TBCCTL6 & CCIFG)==0 ) {};
    // Memorizza valore attuale
    n1 = TBCCR6;
    // Azzera flag di capture e overflow
    TBCCTL6 &= ~(COV|CCIFG);
    // Attende il prossimo evento di capture
    while ( (TBCCTL6 & CCIFG)==0 ) {};
    // Memorizza valore attuale
    n2 = TBCCR6;
    // Se si e' verificato un overflow azzera il valore, se no calcola la differenza
    if ( (TBCCTL6 & COV) != 0 ) n2=0; else n2=n2-n1;
    // Ferma il TimerB
    TBCTL = 0; TBCCTL6 = 0;

#ifdef USE_XT2_FOR_SMCLK_DEFAULT
    // Ripristina il clock a SMCLK da XT2 se necessario
    if (SmclkFromXt2 != 0) BCSCTL2 |= SELS;
#endif
    
    // Ritorna il valore calcolato in KHertz
    return( (n2*32768L)/1000L );
}

#pragma vector=NMI_VECTOR
__interrupt void NMI_ISR(void)
{
  volatile unsigned int i;

  P1OUT |= 0x01;                            // P1.0 = set

  BCSCTL2 &= ~SELM_3;                       // Ensure MCLK runs from DCO
  do
  {
    IFG1 &= ~OFIFG;                         // Clear OSCFault flag
    for (i = 0xFF; i > 0; i--);             // Time for flag to set
  }
  while (IFG1 & OFIFG);                     // OSCFault flag still set?

  BCSCTL2 |= SELM_2;                        // MCLK = XT2 HF XTAL (safe)

  for (i=0; i>20000; i--);                  // Delay for LED on inside ISR
  P1OUT &= ~0x01;                           // P1.0 = reset
}
